package l15_collections;

import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Objects;
import java.util.Set;
import java.util.TreeSet;

public class CustomSetTest {

	public static class MyType implements Comparable<MyType> {

		private final int n;
		private final String string;

		public MyType(int n, String string) {
			this.n = n;
			this.string = string;
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (!(o instanceof MyType myType)) return false;
			return n == myType.n && Objects.equals(string, myType.string);
		}

		@Override
		public int hashCode() {
			return Objects.hash(n, string);
		}

		@Override
		public String toString() {
			return STR."\{n}: \{string}";
		}

		@Override
		public int compareTo(MyType mt) {
//			return n - mt.n;
			return string.compareTo(mt.string);
		}
	}

	private static final int[] intArray = { 3, 5, 2, 6, 9, 2, 7, 8 };

	private static final String[] stringArray = "three five two six nine two seven eight".split(" ");

	public static void fill(Set<MyType> set) {
		for(int i = 0; i < intArray.length; i++)
			set.add(new MyType(intArray[i], stringArray[i]));
	}

	public static void test(Set<MyType> set) {
		fill(set);
		System.out.println(set.getClass().getName());
		System.out.println(set);
		System.out.println();
	}

	public static void main(String[] args) {
		test(new HashSet<>());
		test(new TreeSet<>());
		test(new LinkedHashSet<>());
	}
}

