package l15_collections;

import utilities.RandomGenerators;
import utilities.ThreadHelperMethods;
import java.util.ArrayList;
import java.util.List;
import java.util.Spliterator;
import java.util.concurrent.*;

public class SpliteratorTrySplitExample {
	public static void main (String[] args) {
		int n = 16;
		List<Integer> list = new ArrayList<>();
		for(int i = 0; i < n; ++i) {
			list.add(RandomGenerators.getIntThread(100));
		}
		System.out.println(list);
		System.out.println();

		// trySplit() method efficiently (without traversal) divides its elements in half,
		// allowing balanced parallel computation.
		Spliterator<Integer> s1 = list.spliterator();
		System.out.println(s1.getExactSizeIfKnown()); // 16
		Spliterator<Integer> s2 = s1.trySplit();
		System.out.println(s1.getExactSizeIfKnown()); // 8
		System.out.println(s2.getExactSizeIfKnown()); // 8

		List<Future<Integer>> futures = new ArrayList<>();
		try (ExecutorService executor = Executors.newFixedThreadPool(2)) {
			futures.add(executor.submit(new Adder(s1)));
			futures.add(executor.submit(new Adder(s2)));
		}
		long sum = 0;
		for (Future<Integer> future : futures) {
			try {
				int res = future.get(); // blocks until result available
				System.out.println(res);
				sum += res;
			} catch (InterruptedException | ExecutionException e) {
				e.printStackTrace();
			}
		}
		System.out.println("sum = " + sum);
	}

	static class Adder implements Callable<Integer> {
		private final Spliterator<Integer> spliterator;
		private int sum = 0;

		public Adder(Spliterator<Integer> spliterator) {
			this.spliterator = spliterator;
		}

		@Override
		public Integer call() {

			spliterator.forEachRemaining(i -> {
				sum += i;
				System.out.println(STR."\{Thread.currentThread().getName()}: \{i}");
				ThreadHelperMethods.safeSleep(1000);
			});
			System.out.println(STR."--- \{Thread.currentThread().getName()} got: \{sum}");
			return sum;
		}
	}
}

