package l15_collections;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class SynchronizedArrayListExample {

	static class MutableInteger {
		private int n;

		public MutableInteger(int n) {
			this.n = n;
		}
		public void increment() {
			++n;
		}
		@Override
		public String toString() {
			return String.valueOf(n);
		}
	}

	public static void main(String[] args) {
		final int listSize = 5;
		List<MutableInteger> safeArrayList = new ArrayList<>(listSize);
//		List<MutableInteger> safeArrayList = Collections.synchronizedList(new ArrayList<>(listSize));
		for (int i = 0; i < listSize; i++) {
			safeArrayList.add(new MutableInteger(i));
		}
		System.out.println(safeArrayList);

		try(ExecutorService executorService = Executors.newFixedThreadPool(8)) {
			// Submit the task to the executor service 1000 times.
			// All the tasks will modify the ArrayList concurrently
			for (int i = 0; i < 1000; i++) {
				executorService.submit(() -> incrementArrayList(safeArrayList));
			}
		}
		System.out.println(safeArrayList);
	}

	// Increment all the values in the ArrayList by one
	private static void incrementArrayList(List<MutableInteger> safeArrayList) {
		safeArrayList.forEach(MutableInteger::increment);
	}
}

